from datetime import datetime
from easedate.easedate import *

# TODO:
    # Human-intelligible date
    # https://strftime.org/  apply theses rules

def test_parse_from_dd_mm_yyyy_to_yyyy_mm_dd():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y")
    assert result == parse("2020-03-22")

def test_parse_from_dd_mm_yyyy_to_yy_mm_dd():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%y-%m-%d")
    assert result == parse("20-03-22")

def test_parse_from_dd_mm_yyyy_to_yyyy_mm_dd_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%Y/%m/%d")
    assert result == parse("2020/03/22")

def test_parse_from_dd_mm_yyyy_to_yy_mm_dd_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%y/%m/%d")
    assert result == parse("20/03/22")

def test_parse_from_dd_mm_yyyy_to_mm_dd_yyyy():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%m-%d-%Y")
    assert result == parse("03-22-2020")

def test_parse_from_dd_mm_yyyy_to_mm_dd_yy():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%m-%d-%y")
    assert result == parse("03-22-20")

def test_parse_from_dd_mm_yyyy_to_mm_dd_yyyy_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%m/%d/%Y")
    assert result == parse("03/22/2020")

def test_parse_from_dd_mm_yyyy_to_mm_dd_yy_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%m/%d/%y")
    assert result == parse("03/22/20")

def test_parse_from_dd_mm_yyyy_to_dd_mm_yyyy():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%d-%m-%Y")
    assert result == parse("22-03-2020")

def test_parse_from_dd_mm_yyyy_to_dd_mm_yy():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%d-%m-%y")
    assert result == parse("22-03-20")

def test_parse_from_dd_mm_yyyy_to_dd_mm_yyyy_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%d/%m/%Y")
    assert result == parse("22/03/2020")

def test_parse_from_dd_mm_yyyy_to_dd_mm_yy_slash():
    result = str_to_date("22/03/2020", input_format="%d/%m/%Y", output_format="%d/%m/%y")
    assert result == parse("22/03/20")

def test_today_date():
    assert datetime.now().date() == today()

def test_today_date_name():
    assert today().strftime("%A") == weekday(today())

def test_today_date_name_from_str():
    assert "Wednesday" == weekday("2020-05-20")

def test_weekday_name():
    assert weekday("2020-06-01") == "Monday"

def test_weekday_index():
    assert weekday("2020-06-01", "%w") == "1"

def test_weekday_index_bar():
    assert weekday("22/03/2020", "%w") == "0"

def test_today_date_name_abbreviation():
    assert today().strftime("%a") == weekday(today(), output_format="%a")

def test_add_one_day():
    assert add_day("2020-06-01", 1) == parse("2020-06-02")

def test_add_two_days():
    assert add_day("2020-06-01", 2) == parse("2020-06-03")

def test_subtract_one_day():
    assert add_day("2020-06-02", -1) == parse("2020-06-01")

def test_add_one_month():
    assert add_month("2020-06-01", 1) == parse("2020-07-01")

def test_add_two_months():
    assert add_month("2020-06-01", 2) == parse("2020-08-01")

def test_add_one_year():
    assert add_year("2020-06-01", 1) == parse("2021-06-01")

def test_add_two_years():
    assert add_year("2020-06-01", 2) == parse("2022-06-01")

def test_diff_in_days_zero():
    assert diff_in_days("2020-06-01", "2020-06-01") == 0

def test_diff_in_days_one():
    assert diff_in_days("2020-06-01", "2020-06-02") == 1

def test_diff_in_days_twenty():
    assert diff_in_days("2020-06-01", "2020-06-21") == 20

def test_diff_in_months_zero():
    assert diff_in_months("2020-06-01", "2020-06-01") == 0

def test_diff_in_months_one():
    assert diff_in_months("2020-06-01", "2020-07-01") == 1

def test_diff_in_months_twelve():
    assert diff_in_months("2020-06-01", "2021-06-01") == 12

def test_diff_in_years_zero():
    assert diff_in_years("2020-06-01", "2020-06-01") == 0

def test_diff_in_years_one():
    assert diff_in_years("2020-06-01", "2021-07-01") == 1

def test_diff_in_years_twelve():
    assert diff_in_years("2023-06-01", "2020-06-01") == 3

def test_is_after_by_days_false():
    assert is_after("2020-06-01", "2020-06-02") is False

def test_is_after_by_months_false():
    assert is_after("2020-05-01", "2020-06-01") is False

def test_is_after_by_years_false():
    assert is_after("2019-06-01", "2020-06-01") is False

def test_is_after_by_days_true():
    assert is_after("2020-06-03", "2020-06-02") is True

def test_is_after_by_months_true():
    assert is_after("2020-07-01", "2020-06-01") is True

def test_is_after_by_years_true():
    assert is_after("2021-06-01", "2020-06-01") is True

def test_is_before_by_days_false():
    assert is_before("2020-06-03", "2020-06-02") is False

def test_is_before_by_months_false():
    assert is_before("2020-07-01", "2020-06-01") is False

def test_is_before_by_years_false():
    assert is_before("2021-06-01", "2020-06-01") is False

def test_is_before_by_days_true():
    assert is_before("2020-06-01", "2020-06-02") is True

def test_is_before_by_months_true():
    assert is_before("2020-05-01", "2020-06-01") is True

def test_is_before_by_years_true():
    assert is_before("2019-06-01", "2020-06-01") is True

def test_is_equals_false():
    assert is_equal("2021-06-02", "2020-06-01") is False

def test_is_equals_true():
    assert is_equal("2020-06-01", "2020-06-01") is True

def test_parse_dd_mm_yy_bar():
    assert parse("01/05/2020") == parse("2020-05-01")

def test_parse_dd_mm_yy_bar_inverse():
    assert parse("2020/05/01") == parse("2020-05-01")


"""Detailed descriptions will be added soon.
   At this time the code is enough.
"""

from datetime import datetime
from datetime import timedelta
from dateutil.relativedelta import relativedelta

_MONTHS_IN_YEAR = 12
_TODAY = datetime.date
_FORMATS = [
    '%Y-%m-%d',
    '%y-%m-%d',
    '%Y/%m/%d',
    '%y/%m/%d',
    "%d/%m/%Y",
    "%d-%m-%y",
    "%m/%d/%Y"
]

def _force_parse(str_date):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    for output_format in _FORMATS:
        try:
            return datetime.strptime(str_date, output_format).date()
        except Exception as e:
            # print(f"format: {output_format} error: {e}")
            pass
    return str_date

def _formalize(str_date):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    if isinstance(str_date, str):
        return parse(str_date)
    return str_date

def parse(str_date):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat.
    Final comments will be added soon.
    """
    return _force_parse(str_date)

def str_to_date(str_date, input_format="%Y-%m-%d", output_format="%F"):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    return parse(datetime.strptime(str_date, input_format).strftime(output_format))

def today():
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. 
    Final comments will be added soon.
    """
    return datetime.now().date()

def weekday(date, output_format="%A"):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    date = _formalize(date)
    return date.strftime(output_format)

def add_day(date, value):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    return _formalize(date) + timedelta(days=value)


def add_month(date, value):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    return _formalize(date) + relativedelta(months=value)


def add_year(date, value):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    return _formalize(date) + relativedelta(years=value)


def diff_in_days(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat.
    Final comments will be added soon.
    """
    date = _formalize(date)
    base_date = _formalize(base_date)
    return abs((date - base_date).days)

def diff_in_months(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    from dateutil import relativedelta
    date = _formalize(date)
    base_date = _formalize(base_date)
    result = relativedelta.relativedelta(date, base_date)
    return abs(result.years * _MONTHS_IN_YEAR + result.months)

def diff_in_years(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat.
    Final comments will be added soon.
    """
    date = _formalize(date)
    base_date = _formalize(base_date)
    return abs(date.year - base_date.year)

def is_after(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    date = _formalize(date)
    base_date = _formalize(base_date)
    return date > base_date

def is_before(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat.
    Final comments will be added soon.
    """
    date = _formalize(date)
    base_date = _formalize(base_date)
    return date < base_date

def is_equal(date, base_date = _TODAY):
    """
    Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut 
    labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit 
    esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
    culpa qui officia deserunt mollit anim id est laborum.
    Final comments will be added soon.
    """
    date = _formalize(date)
    base_date = _formalize(base_date)
    return date == base_date






